/***********************************************************************
 * File: simple_hashing
 * 	(Extracted from
 *	   "$Id: simple_hashing.c,v 1.1 2008/08/09 19:45:13 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source implements a simple hashing API-s.
 *
 * Copyright (C) 2005-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "simple_hashing.h" /* simple hashing definitions */


/*
 * This private method is default method to make a hash value from key
 * strings.  If a key string is numeric string, it will be converted
 * to an integer using 'strtol()' before processing the hash value.
 *
 * If several error occurred, this return 'ILLEGAL_HASH_VALUE'.
 * Otherwise, this return 0 or positive integer as hash value.
 *
 * Arguments means...
 *	keys	- pointer to pointer to key strings
 *	n_keys	- size of 'keys' pointer array
 *	depth	- size of the hash table ('depth' member of
 *		  'Hashtable_t')
 */
static ssize_t simple_hash_value(char **keys
				, size_t n_keys
				, ssize_t depth
				)
{
	ssize_t	value, lvalue;
	char	*endptr;

	/* process the every keys... */
	for (endptr = NULL, value = 0
	; ((n_keys > 0) && keys != NULL) && *keys != NULL
	; --n_keys, ++keys) {
		/* try to convert the key string to an integer. */
		errno	= 0; /* initialize errno */
		lvalue	= (ssize_t)strtol((const char *)(*keys)
					, &endptr
					, 10
					);
		if ((*endptr == '\0')
		|| ((lvalue == (ssize_t)LONG_MAX
				|| lvalue == (ssize_t)LONG_MIN)
			&& errno != ERANGE)
		) {
			/* process the numeric key */
			value	^= lvalue;
		}
		else {	/* the key string isn't numeric string. */
			char	*a_key = *keys;

			while (*a_key != '\0') {
				value = (ssize_t)((*a_key++ & 0x7f)
					| (~255L &
						((unsigned)value << 8))
					| (255L & ((unsigned)value >>
						((sizeof(value) - 1)
								* 8
						)))
					);
			}
		}
	}

	if (endptr == NULL) {
		return	(ssize_t)ILLEGAL_HASH_VAL; /* process no keys */
	}

	value &= ~(1 << (sizeof(ssize_t) * 8 - 1)); /* sign-bit off */
	errno = 0; return (value % depth);
				/* round to depth of hash-table */
}


/*
 * This private method tests the hash table structure is sane or not.
 * If the structure is sane, this returns non-zero. Otherwise, this
 * returns zero.
 *
 * Arguments means...
 *	table	- pointer to the hash table
 */
static int is_sane_hash_table(Hashtable_t *table)
{
	return	((((((table != NULL)
		&& table->make_hash_value != NULL)
		&& table->cmp_keys_element != NULL)
		&& table->depth > 0)
		&& table->used_cnt >= 0)?	1: 0);
}


/* This private method returns the minimum value of two integers. */
static ssize_t min(ssize_t a, ssize_t b)
{
	return	(a < b)? a: b;
}


/*
 * This private method returns the least count of extensible datum.
 *
 * The result is equal or less than 'elem_cnt_per_page'.
 */
static ssize_t adjust_allocate_base_cnt(ssize_t	insert_max_cnt
					, ssize_t depth
					, ssize_t elem_cnt_per_page
					)
{
	ssize_t	maxcnt = (insert_max_cnt + depth - 1) / depth;

	return	(maxcnt > 0)?	min(maxcnt, elem_cnt_per_page)
			:	elem_cnt_per_page
			;
}


/*
 * This private method evaluates the datum array allocation hints and
 * sets results to the hash table.
 *
 * If 'hint' is NULL, this set the defaults.
 * The default parameters are...
 *	1) maximum insert datum count	     : LONG_MAX
 *	2) the first allocation datum counts :
 *					datum counts per a memory page.
 *	3) maximum extensible datum counts   :
 *					datum counts per a memory page.
 */
static void eval_hint(Hashtable_t  *result
			, Hashtable_extend_hint_t *hint
			)
{
	/* set the datum count in a memory page */
	result->elem_cnt_per_page	= (ssize_t)(
#			if defined(SC_PAGESIZE) /* { for POSIX */
				sysconf(SC_PAGESIZE)
#			elif defined(NBPG) /* } { for BSD-Unix */
				NBPG
#			else /* } { default */
				HASHTABLE_SYSTEM_PAGESIZE_DEFAULT
#			endif /* } */
			);
	/* test system page size is illegal? */
	if (result->elem_cnt_per_page < 0) {
		result->elem_cnt_per_page =
			(ssize_t)HASHTABLE_SYSTEM_PAGESIZE_DEFAULT;
	}
	result->elem_cnt_per_page
			/= sizeof(result->tables[0].elements[0]);

	/* test hint is nothing? */
	if (hint == NULL) {
		/* set the API default parameters */
		result->max_insert_cnt	= (ssize_t)LONG_MAX;
		result->alloc_base_cnt	= result->alloc_ext_cnt
					= result->elem_cnt_per_page
					;
	}
	else {	/* evaluate the invoker specified hint */
		ssize_t	max_insert_cnt;

		/* set the maximum datum counts */
		max_insert_cnt	= (hint->max_insert_cnt <=
					HASHTABLE_SYS_MAX_INSERT_CNT)?
						  (ssize_t)LONG_MAX
						: hint->max_insert_cnt
						;
		if (result->inserted_cnt < max_insert_cnt) {
			result->max_insert_cnt	= max_insert_cnt;
		}

		/* test invoker rejects datum array extending?  */
		if (hint->elems_ext_cnt == HASHTABLE_NEVER_EXTEND) {
			/* set the first time allocation datum counts */
			result->alloc_base_cnt	= (
				(HASHTABLE_RECOMMENDED_ELEMS_BASE_CNT
					>= hint->elems_base_cnt)?
					  adjust_allocate_base_cnt(
						  result->max_insert_cnt
						, result->depth
						, result->
						       elem_cnt_per_page
						)
					: hint->elems_base_cnt
					);

			/* set the extension datum count to zero */
			result->alloc_ext_cnt = HASHTABLE_NEVER_EXTEND;
		}
		else {	/* invoker needs to extend datum array */
			/* set the first time allocation datum count */
			/* get the system limitation */
			result->alloc_base_cnt	=
					adjust_allocate_base_cnt(
						  result->max_insert_cnt
						, result->depth
						, result->
						    elem_cnt_per_page
						);

			/* test invoker specifies the first allocation
			 * datum counts?
			 */
			if (hint->elems_base_cnt > 0L) {
				/* set to the least datum count of
				 * the system limitation or invoker's
				 * parameter
				 */
				result->alloc_base_cnt	=
					min(hint->elems_base_cnt
						, result->alloc_base_cnt
						);
			}

			/* test invoker needs the recommended extend? */
			if (hint->elems_ext_cnt <= (ssize_t)
				HASHTABLE_RECOMMENDED_ELEMS_EXT_CNT) {
				/* set the recommended extend-count */
				result->alloc_ext_cnt =
					result->elem_cnt_per_page;
			}
			else {	/* invoker specified the extend cnt */
				ssize_t	rest =
					result->max_insert_cnt
					- result->inserted_cnt
					- (result->alloc_base_cnt
						* (result->depth
						- result->used_cnt))
					;
				result->alloc_ext_cnt	=
					min(result->elem_cnt_per_page
						, hint->elems_ext_cnt
						);
				if (rest > 0) {
					result->alloc_ext_cnt	=
						min((rest
							+ result->depth
							- 1)
							 / result->depth
						    , result
						    	->alloc_ext_cnt
						);
				}
			}
		}
	}

	return;
}


/*
 * This private method returns the extensible datum counts.
 * If the hash table never extends the datum array, the result is zero.
 */
static ssize_t get_extend_elem_cnt(Hashtable_t *table)
{
	ssize_t	result;
	
	/* test this hash table never extend the datum array? */
	if (table->alloc_ext_cnt == HASHTABLE_NEVER_EXTEND) {
		result	= 0L; /* no extensible datum */
	}
	else {	/* calculate counts of the extend datum */
		/* get the extend counts according rest datum */
		result	= (table->max_insert_cnt - table->inserted_cnt
				+ table->depth - 1) / table->depth;

		/* limited the result by the page size */
		result	= min(((result <= 0L)?
					  table->elem_cnt_per_page
					: min(table->elem_cnt_per_page
						, result
						)
					)
				, table->alloc_ext_cnt
				);
	}

	return	result;
}


/*
 * This private method finds the datum element that is matched the keys.
 * This invokes user specified 'cmp_keys_elements' compare function.
 * This outputs the index of the datum in 'elements' array, too. The
 * outputted indexes the found datum (return value isn't NULL) or the
 * insert position (return value is NULL and 'errno' is ENOENT).
 *
 * If several errors occurred, this returns NULL and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments.
 *	ENOENT	: nothing is matched keys.
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	keys	- pointer to pointer to key strings
 *	n_keys	- size of the 'keys' pointer array
 *	value	- hash value of 'keys' strings
 *	elements_idx - address to output the index found/insert position
 *	pos_cmp_result - address to output the result of comparing
 *			 'keys' and 'elements_idx' indexing datum
 */
static void *find_element(Hashtable_t	*table
			, char		**keys
			, size_t	n_keys
			, ssize_t	value
			, ssize_t	*elements_idx
			, int		*pos_cmp_result
			)
{
	struct hashtable_elems_t *tables;
	void	*result;

	/* initialize the outputs */
	*elements_idx	= 0; /* insert position is the first */
	*pos_cmp_result	= -1; /* insert the head of 'elements' */

	tables = table->tables + value; /* get the table entry */

	/* search the datum that is matched key strings... */
	if ((tables->duplicated_cnt > 0)
	&& bsearch_array(table->cmp_keys_element
				, (void **)tables->elements
				, tables->duplicated_cnt
				, keys
				, n_keys
				, elements_idx
				, pos_cmp_result
				) != 0) {
		/* found the datum */
		errno	= 0;
		result	= tables->elements[*elements_idx];
	}
	else {	/* not found the datum */
		errno	= ENOENT;
		result	= NULL;
	}

	return	result;	/* found datum address, or NULL (unfounded) */
}


/*
 * This private method insert/replace the datum into 'elements' array
 * pointed from 'tables' member. 'do_replace' argument isn't zero,
 * this will replace the datum.
 * This returns the hash table that is inserted/replaced the datum.
 *
 * If several errors occurred, this returns NULL and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *	ERANGE	: 'make_hash_value' function returns negative integer.
 *	ENOENT	: full of the inserted datum
 *	ENOMEM	: insufficient memories for 'malloc()', or this hash
 *		  table never extend the 'elements' array that was
 *                allocated once.
 *	ESRCH	: the same keys datum was inserted, yet.
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	keys	- pointer to pointer to key strings
 *	n_keys	- size of 'keys' pointer array
 *	elem	- pointer to the inserting datum
 *	do_replace - flag to force replace the datum in hash table
 *		     (it is set non-zero, this replace the datum.
 *		      otherwise, this never replace the datum.)
 */
static Hashtable_t *_insert_hash_table(Hashtable_t *table
				, char	**keys
				, size_t n_keys
				, void	*elem
				, int	do_replace
				)
{
	ssize_t	value;
	struct hashtable_elems_t *tables;
	void	**old_elements, *found_elem;
	ssize_t	elements_idx = 0L;
	int	pos_cmp_result;

	/* test the arguments are illegal? */
	if (((keys == NULL) || elem == NULL)
	|| is_sane_hash_table(table) == 0) {
		errno	= EINVAL;
		return	NULL; /* Invalid arguments */
	}

	/* try to make the hash value from keys */
	if ((value = (*table->make_hash_value)(keys
						, n_keys
						, table->depth
						)) < 0) {
		errno	= ERANGE;
		return	NULL; /* Invalid keys */
	}

	/* find the datum in 'elements' array that is matched keys */
	tables = table->tables + value;
	if ((found_elem = find_element(table
					, keys
					, n_keys
					, value
					, &elements_idx
					, &pos_cmp_result
					)) == NULL) {
		/* unfounded the keys matched datum
		 * ==> insert the datum
		 */
		/* test the hash table or the datum array is full? */
		if (((IS_HASHTABLE_TABLES_FULL(table))
		|| IS_HASHTABLE_ELEMS_ELEMENTS_FULL(tables))
		|| (table->alloc_ext_cnt == HASHTABLE_NEVER_EXTEND
			&& IS_HASHTABLE_ELEMS_ELEMENTS_ALLOCATED(
								tables))
		) {
			errno	= ENOENT;
			return	NULL; /* This hash table is full */
		}

		/* test must the datum be inserted to the first
		 * or the next position of 'found_elem'?
		 */
		if (pos_cmp_result > 0) {
			/* adjust the insert position index */
			++elements_idx;
		}

		/* test 'elements' array has no free elements? */
		if (tables->duplicated_cnt == tables->elements_len) {
			/* create or expand size of 'elements' array */
			ssize_t	alloc_cnt;

			old_elements	= tables->elements;
			alloc_cnt	=
				(old_elements != NULL)?
					  get_extend_elem_cnt(table)
					: table->alloc_base_cnt
					;
			if ((tables->elements = (void **)(malloc(
					sizeof(tables->elements[0])
					* (tables->elements_len
							+ alloc_cnt)
				))) == NULL) {
				/* failed to allocate expanded array */
				tables->elements	= old_elements;
				errno	= ENOMEM;
				return	NULL;
					/* Insufficient memories */
			}
			memset((char *)(tables->elements)
						+ tables->elements_len
				, (int)'\0'
				, (size_t)alloc_cnt
				); /* initialize extended areas */

			/* test the 'elements' is expanded? */
			if (old_elements != NULL) {
				/* move the inserted datum to the
				 * expanded 'elements', and make the
				 * 'elements_idx'-th element to be free
				 */
				ssize_t	n_tail_elems
					= tables->duplicated_cnt
							- elements_idx;

				/* test the head datum exists? */
				if (elements_idx > 0) {
					/* copy the head datum */
					memcpy(tables->elements
						, old_elements
						, sizeof(*old_elements)
							* elements_idx
						);
				}

				/* test the tail datum exists? */
				if (n_tail_elems > 0) {
					/* move the tail datum forward
					 * next position
					 */
					memcpy(tables->elements
							+ (elements_idx
								+ 1)
						, old_elements
							+ elements_idx
						, sizeof(*old_elements)
							* n_tail_elems
						);
				}

				/* release the elder 'elements' array */
				free(old_elements); old_elements = NULL;

				/* adjust the length of 'elements' array
				 */
				tables->elements_len += alloc_cnt;
			}
			else { /* the first allocation this entry */
				/* adjust the length of 'elements' array
				 */
				tables->elements_len
					= table->alloc_base_cnt;
			}
		}
		else if (tables->duplicated_cnt > 0) {
			/* the 'elements' is filled something, yet.
			 * ==> move the tail datum, and make
			 *     'elements_idx'-th element to be free
			 */
			ssize_t	i;

			/* move inserted datum pointers to the next */
			for (i = tables->duplicated_cnt
			; i > elements_idx
			; --i) {
				tables->elements[i]
					= tables->elements[i - 1];
			}
		}

		/* insert datum, and maintain the indexes of 'tables' */
		tables->elements[elements_idx]	= elem;
		if (tables->duplicated_cnt == 0) {
			++(table->used_cnt);
		}
		++(tables->duplicated_cnt);
		++(table->inserted_cnt);
	}
	else if (found_elem != elem) {
		/* the datum is differ the inserted
		 * ==> test invoker needs to replace the datum?
		 */
		if (do_replace != 0) {
			/* replace the inserted to the datum */
			tables->elements[elements_idx]	= elem;
		}
		else {
			/* the same keys ware inserted, yet */
			errno	= ESRCH; /* Yet inserted the keys */
			return	NULL;
		}
	}

	errno = 0; return table;
			/* this returns inserted/replaced hash table */
}


/*
 * This API creates the new hash table structure.
 * Invoker must specify the depth of hash table and the function that
 * compares keys and an element.
 * Invoker is accepted to specify a hash value maker function
 * ('make_hash_value' arg). If invoker doesn't set `make_hash_value',
 * the function is set to the default 'simple_hash_value()'.
 *
 * If 'hint' isn't NULL, the parameters adjusted and set to the several
 * members of 'Hashtable_t'.  Otherwise, this API sets the defaults to
 * those.  The members are referred by the private method
 * '_insert_hash_table()' used by 'insert_hashtable()' and
 * 'replace_hash_table()' API-s.
 *
 * If several errors occurred, this returns NULL. The occurred error
 * detail is indexed by 'errno'.
 *	'errno': means...
 *	EINVAL : illegal arguments
 *	ENOMEM : insufficient memories for 'malloc()'
 *
 * Arguments means...
 *	depth	- size of the creating hash table.
 *	make_hash_value	- pointer to function that makes the hash value
 *			  from key strings.  If it is set NULL, the
 *			  hash table takes the default private method
 *			  'simple_hash_value()'
 *	cmp_keys_element - pointer to function that comparing key
 *			   strings and the datum.  it must be set user
 *			   defined function address.
 *	hint	- pointer to the hint parameters that specifies the
 *		  datum counts for allocation 'elements' array and
 *		  limitation the insert datum.
 */
Hashtable_t *create_hash_table(ssize_t depth
			, ssize_t (*make_hash_value)(char **keys
						, size_t  n_keys
						, ssize_t depth
						)
			, int (*cmp_keys_element)(char	**keys
						, size_t n_keys
						, void	*elem
						)
			, Hashtable_extend_hint_t *hint
			)
{
	Hashtable_t	*result = NULL;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if ((depth <= 0) || cmp_keys_element == NULL) {
		errno = EINVAL; /* Invalid arguments */
	}
	else if ((result = (Hashtable_t *)malloc(
				sizeof(*result)
					+ sizeof(result->tables[0])
						* (depth - 1)
			)) == NULL) {
		/* failed to allocate the hash table area */
		errno	= ENOMEM; /* Insufficient memory */
	}
	else {	/* initialize the new hash table allocated area */
		memset(result
			, (int)'\0'
			, sizeof(result->tables[0]) * (depth - 1)
				+  sizeof(*result)
			);
		result->depth	= depth;
		result->cmp_keys_element = cmp_keys_element;

		/* if invoker doesn't specify the hash value maker
		 * function, this hash table uses the default function
		 * 'simple_hash_value()'.
		 */
		result->make_hash_value
			= (make_hash_value == NULL)?
				  simple_hash_value
				: make_hash_value
				;

		/* evaluate the hint and set the malloc parameters */
		eval_hint(result, hint);

		errno	= 0;
	}

	return	result; /* NULL or allocated area address */
}


/*
 * This API returns size of the allocated ares of the hash table.
 * This returns size that is equal or greater than
 * 'sizeof(Hashtable_t)'.
 *
 * If the several errors occurred, this returns zero and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments (maybe, hash table isn't sane.)
 * Arguments means...
 *	table	- pointer to the hash table
 */
size_t get_allocated_size_hash_table(Hashtable_t *table)
{
	size_t	result = 0UL;
	ssize_t	i, j;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if (is_sane_hash_table(table) == 0) {
		errno	= EINVAL;
		return	result; /* illegal arguments */
	}

	errno	= 0; /* reset 'errno' */

	result = sizeof(*table)
		+ sizeof(table->tables[0]) * (table->depth - 1);
	for (i = 0, j = 0
	; (j < table->used_cnt) && i < table->depth
	; ++i) {
		Hashtable_elems_t *tables = table->tables + i;

		if (IS_HASHTABLE_ELEMS_ELEMENTS_ALLOCATED(tables)) {
			++j; /* counts of calculation used elements */
			result	+= (sizeof(tables->elements[0])
						* tables->elements_len
						);
		}
	}

	return	result; /* greater than sizeof(Hashtable_t) */
}


/*
 * This API returns the counts of the datum elements those were inserted
 * the hash table, yet.
 *
 * If the several errors occurred, this returns ULONG_MAX and set
 * 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments (maybe, hash table isn't sane.)
 * Arguments means...
 *	table	- pointer to the hash table
 */
size_t get_inserted_cnt_hash_table(Hashtable_t *table)
{
	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if (is_sane_hash_table(table) == 0) {
		errno	= EINVAL;
		return	(size_t)ULONG_MAX; /* Invalid arguments */
	}

	errno = 0; return (size_t)(table->inserted_cnt);
				/* counts of inserted datum */
}


/*
 * This API returns the counts of used elements of hash table 'tables'.
 *
 * If the several error occurred, this returns LONG_MIN and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *
 * Arguments means...
 *	table	- pointer to the hash table
 */
ssize_t get_used_cnt_hash_table(Hashtable_t *table)
{
	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if (is_sane_hash_table(table) == 0) {
		errno	= EINVAL; /* Invalid arguments */
		return	(ssize_t)LONG_MIN;
	}

	errno = 0; return table->used_cnt;
		/* counts of the used hash table elements 'tables' */
}


/*
 * This API inserts the datum into 'elements' array of the hash table.
 * This returns the hash table that was inserted the datum.
 *
 * If several errors occurred, this returns NULL and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *	ERANGE	: 'make_hash_value' function returns negative integer.
 *	ENOENT	: full of the inserted datum
 *	ENOMEM	: insufficient memories for 'malloc()', or this hash
 *		  table never extend the 'elements' array that was
 *                allocated once.
 *	ESRCH	: the same keys datum was inserted, yet.
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	keys	- pointer to pointer to key strings
 *	n_keys	- size of 'keys' pointer array
 *	elem	- pointer to the inserting datum
 */
Hashtable_t *insert_hash_table(Hashtable_t *table
				, char **keys
				, size_t n_keys
				, void *elem
				)
{
	errno = 0; return _insert_hash_table(table
					, keys
					, n_keys
					, elem
					, 0 /* don't replace */
					);
}


/*
 * This API replace the datum in 'elements' array of the hash table.
 * If the keys matching datum wasn't inserted yet, this inserts
 * the datum into 'elements' array.
 * This returns the hash table that was replaced/inserted the datum.
 *
 * If several errors occurred, this returns NULL and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *	ERANGE	: 'make_hash_value' function returns negative integer.
 *	ENOENT	: full of the inserted datum
 *	ENOMEM	: insufficient memories for 'malloc()', or this hash
 *		  table never extend the 'elements' array that was
 *                allocated once.
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	keys	- pointer to pointer to key strings
 *	n_keys	- size of 'keys' pointer array
 *	elem	- pointer to the replacing datum
 */
Hashtable_t *replace_hash_table(Hashtable_t *table
					, char **keys
					, size_t n_keys
					, void *elem
					)
{
	errno = 0; return _insert_hash_table(table
					, keys
					, n_keys
					, elem
					, 1 /* do replace */
					);
}


/*
 * This API returns the counts of datum in the same 'elements' array.
 *
 * If several errors occurred, this return LONG_MIN and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *	ERANGE	: 'make_hash_value' function returns negative integer.
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	keys	- pointer to pointer to key strings
 *	n_keys	- size of 'keys' pointer array
 */
ssize_t get_duplicated_cnt_hash_table(Hashtable_t *table
					, char **keys
					, size_t n_keys
					)
{
	ssize_t	value;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if (((keys == NULL) || n_keys == 0)
	|| is_sane_hash_table(table) == 0) {
		errno	= EINVAL;
		return	(ssize_t)LONG_MIN; /* Invalid arguments */
	}

	/* try to make the hash value from keys... */
	if ((value = (*table->make_hash_value)(keys
						, n_keys
						, table->depth
						)) < 0) {
		/* failed in making hash value from keys */
		errno	= ERANGE;
		return	(ssize_t)LONG_MIN; /* out of range (keys) */
	}

	errno = 0; return table->tables[value].duplicated_cnt;
			/* counts of datum in the 'elements' array */
}


/*
 * This API returns the datum in the hash table that is matched
 * the keys.
 *
 * If several errors occurred, this returns NULL and set "errno".
 *	errno	: means
 *	EINVAL	: illegal arguments
 *	ERANGE	: 'make_hash_value' function returns negative integer.
 *	ENOENT	: nothing is matched keys.
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	keys	- pointer to pointer to the key strings
 *	n_keys	- size of 'keys' pointer array
 */
void *get_elem_hash_table(Hashtable_t *table
			, char **keys
			, size_t n_keys
			)
{
	ssize_t	value;
	ssize_t	elements_idx = 0L;
	int	pos_cmp_result;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if (((keys == NULL) || n_keys == 0)
	|| is_sane_hash_table(table) == 0) {
		errno	= EINVAL;
		return	NULL; /* Invalid arguments */
	}

	/* try to make the hash value from keys... */
	if ((value = (*table->make_hash_value)(keys
						, n_keys
						, table->depth
						)) < 0) {
		/* 'make_hash_value' function returns negative */
		errno	= ERANGE;
		return	NULL; /* Invalid keys */
	}

	/* try to find the datum that is matched to keys... */
	errno = 0; return find_element(table
					, keys
					, n_keys
					, value
					, &elements_idx
					, &pos_cmp_result
					);
}


/*
 * This API deletes the datum from the hash table 'elements' array.
 * This returns the hash table that is deleted the datum.
 *
 * If several errors occurred, this returns NULL and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *	ERANGE	: 'make_hash_value' function returns negative integer.
 *	ENOENT	: unfounded the datum in the hash table
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	keys	- pointer to pointer to the key strings
 *	n_keys	- size of 'keys' pointer array
 *	elem	- pointer to the deleting datum
 */
Hashtable_t *delete_hash_table(Hashtable_t *table
					, char **keys
					, size_t n_keys
					, void *elem
					)
{
	ssize_t	value;
	ssize_t	elements_idx = 0L, rest_elements_cnt;
	struct hashtable_elems_t *tables;
	int	pos_cmp_result;
	void	*found_elem;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if ((keys == NULL) || is_sane_hash_table(table) == 0) {
		errno	= EINVAL;
		return	NULL; /* Invalid arguments */
	}

	/* try to make the hash value from keys... */
	if ((value = (*table->make_hash_value)(keys
						, n_keys
						, table->depth
						)) < 0) {
		/* 'make_hash_value' function returns negative. */
		errno	= ERANGE;
		return	NULL; /* Invalid keys */
	}

	/* try to find the datum in the hash table... */
	if (((found_elem = find_element(table
					, keys
					, n_keys
					, value
					, &elements_idx
					, &pos_cmp_result
					)) == NULL)
	|| ((elem != NULL) && found_elem != elem)) {
		/* unfounded the datum in the hash table */
		errno	= ENOENT;
		return	NULL; /* The element wasn't inserted */
	}

	/* maintain the 'elements' array */
	rest_elements_cnt =
		table->tables[value].duplicated_cnt - elements_idx - 1;
	tables	= table->tables + value;
	if (tables->duplicated_cnt > elements_idx + 1) {
		void	*dst = tables->elements + elements_idx
			, *src = tables->elements + (elements_idx + 1);

		memcpy(dst, src
			, (sizeof(*(tables->elements))
						* rest_elements_cnt)
			);
	}
	tables->elements[tables->duplicated_cnt - 1] = NULL;
	--(tables->duplicated_cnt);
	if (tables->duplicated_cnt == 0) {
		--(table->used_cnt);
	}
	--(table->inserted_cnt);

	errno = 0; return table;
			/* hash table that is deleted the datum */
}


/*
 * This API provides the cursor mechanism for the hash table.
 * The cursor will be used as iterator for the hash table.
 * This returns the current datum address of 'elements' in the 'tables'
 * member of 'table'.
 * When '*cursor' equals NULL, this allocates memory of the new cursor,
 * set the current position to the first datum in the hash table and
 * returns the first datum address.
 * When 'direct' equals 'HASHTABLE_CURSOR_DIR_DESTROY' this returns
 * 'table' address.
 *
 * If several errors occurred, this returns NULL and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *	ENOMEM	: insufficient memory
 *
 * Arguments means...
 *	table	- pointer to the hash table
 *	cursor	- address to output the created cursor, or pointer to
 *		  the cursor
 *	direct	- switch to action of this API.
 *			direct	: action of API
 *			HASHTABLE_CURSOR_DIR_CURRENT  :
 *				return the current datum
 *			HASHTABLE_CURSOR_DIR_PREVIOUS :
 *				return the previous datum or NULL
 *			HASHTABLE_CURSOR_DIR_NEXT     :
 *				return the next datum or NULL
 *			HASHTABLE_CURSOR_DIR_DESTROY  :
 *				release resources of the cursor
 */
void *get_elem_by_hashtable_cursor(Hashtable_t			*table
					, Hashtable_cursor_t	**cursor
					, Hashtable_cursor_dir_t direct
					)
{
	void	*result = NULL;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if ((((is_sane_hash_table(table) == 0)
	|| table->inserted_cnt == 0)
	|| cursor == NULL)
	|| ((*cursor != NULL)
		&& (((((((((((table != (*cursor)->table)
			|| table->depth - 1 < (*cursor)->tables_idx)
			|| (*cursor)->tables_idx < 0)
			|| (*cursor)->walked_tables_cnt
				> table->used_cnt)
			|| (*cursor)->walked_tables_cnt < 0)
			|| (*cursor)->elements_idx
				> table->tables[(*cursor)->tables_idx]
					.duplicated_cnt - 1)
			|| (*cursor)->elements_idx < 0)
			|| direct < HASHTABLE_CURSOR_DIR_CURRENT)
			|| direct > HASHTABLE_CURSOR_DIR_DESTROY)
			|| (((direct == HASHTABLE_CURSOR_DIR_PREVIOUS)
				&& (*cursor)->tables_idx == 0)
				&& (*cursor)->elements_idx == 0)
				)
			|| (((direct == HASHTABLE_CURSOR_DIR_NEXT)
				&& (*cursor)->walked_tables_cnt
					== table->used_cnt)
				&& (*cursor)->elements_idx
					== table->tables[
						(*cursor)->tables_idx]
						.duplicated_cnt - 1)
		))
	) {
		errno	= EINVAL;
		return	result; /* illegal arguments */
	}

	/* which action is needed? */
	if (direct == HASHTABLE_CURSOR_DIR_DESTROY) {
		/* release the allocated area for this cursor */
		free(*cursor); *cursor	= NULL;

		/* return the 'table' */
		result	= (void *)table;
	}
	else if (*cursor == NULL) {
		/* make a new cursor for this hash table */
		/* allocate and initialize the new cursor */
		if ((*cursor = (Hashtable_cursor_t *)
				malloc(sizeof(**cursor))) == NULL) {
			errno	= ENOMEM;
			return	result; /* insufficient memory */
		}

		/* set 'table' member */
		(*cursor)->table	= table;

		/* set the first available 'tables' element index
		 * to 'tables_idx' member
		 */
		for ((*cursor)->tables_idx = 0
		; table->tables[(*cursor)->tables_idx].duplicated_cnt
								<= 0
		; ++((*cursor)->tables_idx)) ;

		/* set 'walked_tables_cnt' member */
		(*cursor)->walked_tables_cnt	= 1;

		/* set 'elements_idx' member */
		(*cursor)->elements_idx	= 0;

		/* return the first datum (it's the current datum) */
		goto SET_CURRENT_DATUM_RESULT;
	}
	else if (direct == HASHTABLE_CURSOR_DIR_PREVIOUS) {
		/* move the cursor to the previous datum position */
		if ((*cursor)->elements_idx > 0) {
			/* the previous datum exists in the current
			 * 'elements' array
			 */
			--((*cursor)->elements_idx);
		}
		else {	/* search the previous available 'tables'
			 * element
			 */
			ssize_t	i = (*cursor)->tables_idx;

			while ((--i >= 0)
			&& table->tables[i].duplicated_cnt <= 0) ;
			if (i < 0) {
				errno	= EINVAL;
				return result; /* illegal arguments */
			}

			/* set the next datum position */
			(*cursor)->tables_idx	= i;
			--((*cursor)->walked_tables_cnt);
			(*cursor)->elements_idx =
				table->tables[(*cursor)->tables_idx]
					.duplicated_cnt - 1;
		}

		/* return the previous datum (it's the current datum) */
		goto SET_CURRENT_DATUM_RESULT;
	}
	else if (direct == HASHTABLE_CURSOR_DIR_NEXT) {
		/* move the cursor to the next datum position */
		if (table->tables[(*cursor)->tables_idx]
			.duplicated_cnt - 1
					> (*cursor)->elements_idx) {
			/* the next datum exists in the current
			 * 'elements' array
			 */
			++((*cursor)->elements_idx);
		}
		else {	/* search the next available 'tables' elements
			 */
			while (table->tables[++((*cursor)->tables_idx)]
					.duplicated_cnt <= 0) ;
			++((*cursor)->walked_tables_cnt);
			(*cursor)->elements_idx	= 0;
		}

		/* return the next datum (it's the current datum) */
		goto SET_CURRENT_DATUM_RESULT;
	}
	else  {	/* return the current datum */
SET_CURRENT_DATUM_RESULT:
		result	= table->tables[(*cursor)->tables_idx]
				.elements[(*cursor)->elements_idx];
	}

	errno = 0; return result;
			/* the current datum or 'table (when DESTROY) */
}


/*
 * This API release the allocated areas around the hash table.
 * This returns zero, when the hash table resources are released,
 * completely.
 *
 * If several errors occurred, this returns -1 and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *
 * Arguments means...
 *	table	- pointer to the hash table
 */
int destroy_hash_table(Hashtable_t *table)
{
	ssize_t tablesidx;
	struct hashtable_elems_t *tables;

	errno	= 0; /* initialize errno */

	/* test the arguments are illegal? */
	if (is_sane_hash_table(table) == 0) {
		errno	= EINVAL;
		return -1; /* Invalid argument */
	}

	/* release the 'elements' array allocated areas... */
	for (tablesidx = 0, tables = table->tables
	; tablesidx < table->depth
	; ++tables, ++tablesidx) {
		if (IS_HASHTABLE_ELEMS_ELEMENTS_ALLOCATED(tables)) {
			/* release the 'elements' array area */
			free(tables->elements);
			tables->elements = NULL;
			tables->elements_len
				= tables->duplicated_cnt
				= 0UL
				;
		}
	}

	/* clean the index and counters */
	table->make_hash_value	= NULL;
	table->cmp_keys_element	= NULL;
	table->depth = table->used_cnt = 0L;
	table->inserted_cnt = 0UL;

	/* release the hash table allocated area */
	free(table);

	errno = 0; return 0;
			/* released the hash table allocated areas */
}


/*
 * This API provides the binary search method.
 * This invokes user specified compare function 'cmp_keys_elements'.
 * If the datum is found, this return non-zero.
 * Otherwise, this returns zero (if this gets illegal arguments.
 * 'errno" is set EINVAL).
 * This outputs the index of the found/insert datum in the array.
 *
 * If several errors occurred, this returns zero and set 'errno'.
 *	errno	: means...
 *	EINVAL	: illegal arguments
 *
 * Arguments means...
 *	cmp_keys_elements - pointer to function that comparing key
 *			    strings and the datum in sorted array
 *	array	- pointer to array of sorted datum
 *	n_array	- size of 'array' sorted datum array
 *	keys	- pointer to key strings array
 *	n_keys	- size of 'keys' string array
 *	pos	- address to output the index of found/inserting datum
 *		  position in the 'array'
 *	pos_cmp_result - address to output the result of comparing
 *			 'keys' strings and datum is indexed from
 *			 '*pos'
 */
int bsearch_array(int		(*cmp_keys_elements)(char **keys
							, size_t n_keys
							, void *elem
							)
		, void		**array
		, ssize_t	n_array
		, char		**keys
		, size_t	n_keys
		, ssize_t	*pos
		, int		*pos_cmp_result
		)
{
	ssize_t hi, mid, lo;
	int	cmp;

	errno	= 0; /* initialize errno */

	/* test illegal arguments? */
	if (((((((cmp_keys_elements == NULL)
	|| array == NULL)
	|| n_array < 0L)
	|| keys == NULL)
	|| n_keys < 1)
	|| pos == NULL)
	|| pos_cmp_result == NULL) {
		errno	= EINVAL;
		return	0; /* illegal arguments */
	}

	errno	= 0; /* reset errno */

	/* search the matched/inserting position... */
	for (lo = 0, hi = n_array - 1; lo <= hi; ) {
		mid	= (ssize_t)((((size_t)(hi) + (size_t)lo)) >> 1);
			/* "(lo + hi) / 2" causes "integer range
			 * overflow" bug! The best way to fix the bug is
			 * "(((size_t)(hi) + (size_t)lo)) >> 1" or
			 * "((hi - lo) / 2) + lo"	(FIXED20060625)
			 */
		if ((cmp = (*cmp_keys_elements)(keys
					, n_keys
					, array[mid]
					)) == 0) {
			/* find the matched position in array */
			hi	= mid;
			break;
		}
		else if (cmp < 0) {
			/* keys is least than this position */
			hi	= mid - 1;
		}
		else {
			/* keys is greater than this position */
			lo	= mid + 1;
		}
	}

	/* output the found/inserting position ('hi' indexes) to
	 * the 'pos' pointing area
	 */
	if (hi < 0) {
		*pos	= 0;
		*pos_cmp_result	= INT_MIN;
		cmp	= 0;
	}
	else {
		*pos	= hi;
		cmp	= (((*pos_cmp_result = (*cmp_keys_elements)(keys
							, n_keys
							, array[hi]
							)
						) == 0)?
					1: 0
				);
	}

	/* If the matched element in 'array' is found, output zero to
	 * 'pos_cmp_result' address and return non-zero.
	 * Otherwise, output the result of comparing the keys and
	 * the insert pos ('pos' address indexes) element into the
	 * 'pos_cmp_result' and return zero.
	 */
	return	cmp;
}


/*
 * The followings implements the unit test codes for this API-s.
 * If 'UnitTest' macro is defined, this unit test is enable.
 */
#if defined(UnitTest) /* { Unit Test program */

/*
 * This unit test method makes the hash value from keys,
 * using 'simple_hash_value()' that is defined above.
 *
 * Arguments means...
 *	keys	- pointer to the key string array
 *	n_keys	- size of 'keys' string array
 *	depth	- size of the hash value
 */
ssize_t make_hashval(char **keys, size_t n_keys, ssize_t depth)
{
	ssize_t	result;

	result = simple_hash_value(keys, n_keys, depth);

	return	result;
}

#endif /* } defined(UnitTest) Unit Test program */
